/****************************************************************************
 *  
 * Project: Olimex AVR ISP with STK500 version 2 protocol support.
 *
 * Copyright by Olimex Ltd. All rights reserved.
 * 
 * File: timeout.h
 * Description: Timeout timer routines (kind of software watchdog)
 * Developer: Dimitar Dimitrov ( dinuxbg __on__ gmail.com )
 *
 * Last change: $Date: 2007-12-17 15:36:10 +0200 (понеделник, 17 Декември 2007) $
 * Revision: $Revision: 91 $
 * Id: $Id: timeout.h 91 2007-12-17 13:36:10Z dimitar $
 *
 ****************************************************************************/

#ifndef TIMEOUT_H
#define TIMEOUT_H

#include <util/delay.h>

/** Stop and reset TIMER. */
static void timer_reset(void)
{
	TCCR1B = 0;
	TCNT1 = 0;
	TIFR1 = (1<<TOV1);
}

/** Return non-zero value if the given time interval has passed. */
static uint8_t timer_ready(void)
{
	return TIFR1 & (1<<TOV1);
}

/**
 * Start TIMER and expect overflow in about t ms.
 */
static void timer_start(uint16_t t)
{
	TCCR1B = (1<<CS12) | (0<<CS11) | (1<<CS10);
	TCNT1 = 0xFFFF - t*(F_CPU/(1024UL*1000UL)+1);
	TIFR1 = (1<<TOV1);
}


#endif /* TIMEOUT_H */
