/****************************************************************************
 *  
 * Project: Olimex AVR ISP with MKII protocol support.
 *
 * Copyright by Olimex Ltd. All rights reserved.
 * 
 * File: usb_conf.h
 * Description: USB Configuration Constants
 * Developer: Dimitar Dimitrov ( dinuxbg __on__ gmail.com )
 *
 * Last change: $Date: 2007-12-20 17:24:34 +0200 (четвъртък, 20 Декември 2007) $
 * Revision: $Revision: 97 $
 * Id: $Id: usb_conf.h 97 2007-12-20 15:24:34Z dimitar $
 *
 ****************************************************************************/

#ifndef USB_CONF_H
#define USB_CONF_H

#include "usb_core.h"
#include "massbulk.h"

/** 
 * Set to nonzero to enable strict USB code. Setting this will also increase
 * the code size.
 */
#define USB_STRICT_STANDARD			0

/** Use Olimex' own vendor ID. */
#define USB_VENDOR_ID				0x15ba
#define USB_PRODUCT_ID				0x0012

/** Device release number. */
#define USB_DEV_RELNUM				0x0100

/** Endpoint zero maximal packet size. */
#define USB_CTRLEP_MAX_PACKET_SIZE		16

/** Needed power by the USB device, in mA. */
#define USB_NEEDED_POWER_MA			100

/** Check whether we're selfpowered. 
 */
#define usb_is_self_powered()			(0)

/** Return non-zero value if USB BUS power is detected. */
#define usb_check_bus_power()			(1)

/** Total number of endpoints (including the control endpoint!). */
#define USB_EPN_TOTAL				5

/** Mass storage IN bulk endpoint. */
#define USB_EPN_MS_IN				1

/** Mass storage OUT bulk endpoint. */
#define USB_EPN_MS_OUT				2

/** Endpoint max packet size */
#define USB_EP_MS_IN_MAX_PACKET_SIZE		64
/** Endpoint max packet size */
#define USB_EP_MS_OUT_MAX_PACKET_SIZE		64

/** Set the Mass Storage interface number. */
#define MS_INTRFN				0

/** 
 * Configuration block descriptor, returned when a configuration descriptor
 * is requested.
 */
struct usb_configuration_block {
	struct usb_configuration_descriptor conf;
	struct usb_interface_descriptor interf;
	struct usb_endpoint_descriptor ep_in;
	struct usb_endpoint_descriptor ep_out;
};

/* 
 * User-defined callback functions, called by the USB driver on the 
 * corresponding USB events.
 */

/** 
 * Handle non-standard SETUP requests.
 *
 * @return Zero value for success, non-zero value for error.
 */
extern int8_t ms_usbcb_setupreq(const struct usb_setup_packet *);
#define usbcb_setupreq(A)	ms_usbcb_setupreq(A)

/** Propagate GET_Descriptor standard request to interfaces. */
#define usbcb_get_descriptor(A)		

/** 
 * Propagate the Start-Of-Frame interrupt to the user. 
 *
 * @warning This is executed inside the USB interrupt handler!!!
 */

#define usbcb_sof()	(void)0;

/** 
 * Propagate the suspend interrupt to the user. 
 *
 * @warning This is executed inside the USB interrupt handler!!!
 */
extern void usbcb_suspend(void);

/** 
 * Propagate the wakeup interrupt to the user. 
 *
 * @warning This is executed inside the USB interrupt handler!!!
 */
extern void usbcb_wakeup(void);

/** Configure vendor/class specific endpoints. */
extern void ms_usbcb_set_configuration(void);
#define usbcb_set_configuration()	ms_usbcb_set_configuration()


#endif /* USB_CONF_H */
