/****************************************************************************
 *  
 * Project: Olimex AVR ISP with STK500 version 2 protocol support.
 *
 * Copyright by Olimex Ltd. All rights reserved.
 * 
 * File: swspi.h
 * Description: Software SPI implementation.
 * Developer: Dimitar Dimitrov ( dinuxbg@gmail.com )
 *
 * Last change: $Date: 2007-12-11 17:38:11 +0200 (вторник, 11 Декември 2007) $
 * Revision: $Revision: 86 $
 * Id: $Id: swspi.h 86 2007-12-11 15:38:11Z dimitar $
 *
 ****************************************************************************/

#ifndef SWSPI_H
#define SWSPI_H

#include "board.h"


#define SWSPI_MISO()	(PINR_SWSPI & (1<<PIN_SWSPI_MISO))

#define SWSPI_MOSI(L)	if(L) { PORTR_SWSPI |= (1<<PIN_SWSPI_MOSI); } else { PORTR_SWSPI &= ~(1<<PIN_SWSPI_MOSI); }

#define SWSPI_SCK(L)	if(L) { PORTR_SWSPI |= (1<<PIN_SWSPI_SCK); } else { PORTR_SWSPI &= ~(1<<PIN_SWSPI_SCK); }

#define SWSPI_CS(L)	if(L) { PORTR_SWSPI |= (1<<PIN_SWSPI_CS); } else {PORTR_SWSPI &= ~(1<<PIN_SWSPI_CS); }

#ifdef PIN_SWSPI_RST
  #define SWSPI_RST(L)	if(L) { PORTR_SWSPI |= (1<<PIN_SWSPI_RST); } else {PORTR_SWSPI &= ~(1<<PIN_SWSPI_RST); }
#else
  #define SWSPI_RST(L)	
#endif

#ifdef PIN_SWSPI_WP
  #define SWSPI_WP(L)	if(L) { PORTR_SWSPI |= (1<<PIN_SWSPI_WP); } else {PORTR_SWSPI &= ~(1<<PIN_SWSPI_WP); }
#else
  #define SWSPI_WP(L)	
#endif

/**
 *  @brief Initialize the software spi interface. 
 */
static void swspi_init(void) 
{
	/* set outputs */
	DDR_SWSPI |= (1<<PIN_SWSPI_MOSI) | (1<<PIN_SWSPI_SCK) 
		| (1<<PIN_SWSPI_CS)
#ifdef PIN_SWSPI_RST
	       	| (1<<PIN_SWSPI_RST)
#endif
#ifdef PIN_SWSPI_WP
		| (1<<PIN_SWSPI_WP);
#else
		;
#endif
	DDR_SWSPI &= ~(1<<PIN_SWSPI_MISO);
	/* set default output values and disable MISO pullup */
	PORTR_SWSPI &= ~((1<<PIN_SWSPI_MOSI) | (1<<PIN_SWSPI_SCK) 
			| (1<<PIN_SWSPI_MISO));
	SWSPI_WP(0);
	SWSPI_RST(0);
	_delay_ms(10.0);
	SWSPI_RST(1);
	_delay_ms(10.0);
	/* Toggle CS in order to reset SPI slave. */
	SWSPI_CS(1);
	SWSPI_CS(0);
	SWSPI_CS(1);
}

/**
 * @brief Exchange one data byte using the software SPI interface.
 * @param b Byte to transmig.
 * @return The byte received from SPI slave.
 */
static uint8_t swspi_xdata(uint8_t b)
{
	uint8_t i;

	for(i=0; i<8; ++i) {
		SWSPI_MOSI(b&0x80);
		SWSPI_SCK(1);
		b = b<<1;
		if(SWSPI_MISO()) b |= 1;
		SWSPI_SCK(0);
	}
	return b;
}

#else
  #error "File swspi.h must be included only from C modules that use it!"
#endif /* SWSPI_H */
